@@ -14,6 +14,10 @@
 #include <utmpx.h>
 #include <arpa/inet.h>
 
+#include <mach/mach.h>
+#include <mach/mach_vm.h>
+#include <mach/shared_region.h>
+
 #include "ps-internal.h"
 #include "arch/macos/process_info.h"
 
@@ -45,12 +49,13 @@
   default:     error;					\
   }
 
-void ps__check_for_zombie(ps_handle_t *handle) {
+void ps__check_for_zombie(ps_handle_t *handle, int err) {
   struct kinfo_proc kp;
   int ret;
 
   if (handle->pid == 0) {
     ps__access_denied("");
+    err = 1;
 
   } else if (errno == 0 || errno == ESRCH) {
 
@@ -58,8 +63,10 @@ void ps__check_for_zombie(ps_handle_t *handle) {
     if ((ret == -1) ||
 	(PS__TV2DOUBLE(kp.kp_proc.p_starttime) != handle->create_time)) {
       ps__no_such_process(handle->pid, 0);
+      err = 1;
     } else if (kp.kp_proc.p_stat == SZOMB) {
       ps__zombie_process(handle->pid);
+      err = 1;
     } else {
       ps__access_denied("");
     }
@@ -68,7 +75,7 @@ void ps__check_for_zombie(ps_handle_t *handle) {
     ps__set_error_from_errno();
   }
 
-  ps__throw_error();
+  if (err) ps__throw_error();
 }
 
 void psll_finalizer(SEXP p) {
@@ -206,7 +213,7 @@ SEXP psll_exe(SEXP p) {
 
   ret = proc_pidpath(handle->pid, &buf, sizeof(buf));
 
-  if (ret == 0) ps__check_for_zombie(handle);
+  if (ret == 0) ps__check_for_zombie(handle, 1);
 
   PS__CHECK_HANDLE(handle);
 
@@ -221,7 +228,7 @@ SEXP psll_cmdline(SEXP p) {
 
   result = ps__get_cmdline(handle->pid);
 
-  if (isNull(result)) ps__check_for_zombie(handle);
+  if (isNull(result)) ps__check_for_zombie(handle, 1);
 
   PROTECT(result);
   PS__CHECK_HANDLE(handle);
@@ -280,7 +287,7 @@ SEXP psll_cwd(SEXP p) {
 
   if (ps__proc_pidinfo(handle->pid, PROC_PIDVNODEPATHINFO, 0, &pathinfo,
 		       sizeof(pathinfo)) <= 0) {
-    ps__check_for_zombie(handle);
+    ps__check_for_zombie(handle, 1);
   }
 
   PS__CHECK_HANDLE(handle);
@@ -360,7 +367,7 @@ SEXP psll_environ(SEXP p) {
 
   result = ps__get_environ(handle->pid);
 
-  if (isNull(result)) ps__check_for_zombie(handle);
+  if (isNull(result)) ps__check_for_zombie(handle, 1);
 
   PROTECT(result);
   PS__CHECK_HANDLE(handle);
@@ -378,7 +385,7 @@ SEXP psll_num_threads(SEXP p) {
 
   if (ps__proc_pidinfo(handle->pid, PROC_PIDTASKINFO, 0, &pti,
 		       sizeof(pti)) <= 0) {
-    ps__check_for_zombie(handle);
+    ps__check_for_zombie(handle, 1);
   }
 
   PS__CHECK_HANDLE(handle);
@@ -396,7 +403,7 @@ SEXP psll_cpu_times(SEXP p) {
 
   if (ps__proc_pidinfo(handle->pid, PROC_PIDTASKINFO, 0, &pti,
 		       sizeof(pti)) <= 0) {
-    ps__check_for_zombie(handle);
+    ps__check_for_zombie(handle, 1);
   }
 
   PS__CHECK_HANDLE(handle);
@@ -423,7 +430,7 @@ SEXP psll_memory_info(SEXP p) {
 
   if (ps__proc_pidinfo(handle->pid, PROC_PIDTASKINFO, 0, &pti,
 		       sizeof(pti)) <= 0) {
-    ps__check_for_zombie(handle);
+    ps__check_for_zombie(handle, 1);
   }
 
   PS__CHECK_HANDLE(handle);
@@ -466,17 +473,17 @@ SEXP ps__cpu_count_logical() {
 
   if (sysctlbyname("hw.logicalcpu", &num, &size, NULL, 2))
     return ScalarInteger(NA_INTEGER);
-  else 
+  else
     return ScalarInteger(num);
 }
 
 SEXP ps__cpu_count_physical() {
   int num = 0;
   size_t size = sizeof(int);
-  
+
   if (sysctlbyname("hw.physicalcpu", &num, &size, NULL, 0))
     return ScalarInteger(NA_INTEGER);
-  else 
+  else
     return ScalarInteger(num);
 }
 
@@ -569,7 +576,7 @@ SEXP psll_num_fds(SEXP p) {
   pid = handle->pid;
 
   pidinfo_result = proc_pidinfo(pid, PROC_PIDLISTFDS, 0, NULL, 0);
-  if (pidinfo_result <= 0) ps__check_for_zombie(handle);
+  if (pidinfo_result <= 0) ps__check_for_zombie(handle, 1);
 
   fds_pointer = malloc(pidinfo_result);
   if (fds_pointer == NULL) {
@@ -582,7 +589,7 @@ SEXP psll_num_fds(SEXP p) {
 
   if (pidinfo_result <= 0) {
     free(fds_pointer);
-    ps__check_for_zombie(handle);
+    ps__check_for_zombie(handle, 1);
   }
 
   num = (pidinfo_result / PROC_PIDLISTFD_SIZE);
@@ -669,7 +676,7 @@ SEXP psll_open_files(SEXP p) {
 
  error:
   if (fds_pointer != NULL) free(fds_pointer);
-  ps__check_for_zombie(handle);
+  ps__check_for_zombie(handle, 1);
   return R_NilValue;
 }
 
@@ -805,10 +812,128 @@ SEXP psll_connections(SEXP p) {
 
  error:
   if (fds_pointer) free(fds_pointer);
-  ps__check_for_zombie(handle);
+  ps__check_for_zombie(handle, 1);
   return R_NilValue;
 }
 
+/*
+ * Indicates if the given virtual address on the given architecture is in the
+ * shared VM region.
+ */
+static bool ps_in_shared_region(mach_vm_address_t addr, cpu_type_t type) {
+  mach_vm_address_t base;
+  mach_vm_address_t size;
+
+  switch (type) {
+  case CPU_TYPE_ARM:
+    base = SHARED_REGION_BASE_ARM;
+    size = SHARED_REGION_SIZE_ARM;
+    break;
+  case CPU_TYPE_I386:
+    base = SHARED_REGION_BASE_I386;
+    size = SHARED_REGION_SIZE_I386;
+    break;
+  case CPU_TYPE_X86_64:
+    base = SHARED_REGION_BASE_X86_64;
+    size = SHARED_REGION_SIZE_X86_64;
+    break;
+  default:
+    return false;
+  }
+
+  return base <= addr && addr < (base + size);
+}
+
+/*
+ * Returns the USS (unique set size) of the process. Reference:
+ * https://dxr.mozilla.org/mozilla-central/source/xpcom/base/
+ *     nsMemoryReporterManager.cpp
+ */
+
+SEXP psll_memory_uss(SEXP p) {
+  ps_handle_t *handle = R_ExternalPtrAddr(p);
+  long pid;
+  size_t len;
+  cpu_type_t cpu_type;
+  size_t private_pages = 0;
+  mach_vm_size_t size = 0;
+  mach_msg_type_number_t info_count = VM_REGION_TOP_INFO_COUNT;
+  kern_return_t kr;
+  long pagesize = getpagesize();
+  mach_vm_address_t addr = MACH_VM_MIN_ADDRESS;
+  mach_port_t task = MACH_PORT_NULL;
+  vm_region_top_info_data_t info;
+  mach_port_t object_name;
+
+  if (!handle) error("Process pointer cleaned up already");
+  pid = handle->pid;
+
+  kern_return_t err = KERN_SUCCESS;
+  err = task_for_pid(mach_task_self(), pid, &task);
+  if (err != KERN_SUCCESS) {
+    ps__check_for_zombie(handle, 1);
+    PS__CHECK_HANDLE(handle);
+    ps__set_error("Access denied for task_for_pid() for %d", (int) pid);
+    ps__throw_error();
+  }
+
+  len = sizeof(cpu_type);
+  if (sysctlbyname("sysctl.proc_cputype", &cpu_type, &len, NULL, 0) != 0) {
+    ps__set_error_from_errno();
+    ps__throw_error();
+  }
+
+  // Roughly based on libtop_update_vm_regions in
+  // http://www.opensource.apple.com/source/top/top-100.1.2/libtop.c
+  for (addr = 0; ; addr += size) {
+    kr = mach_vm_region(
+      task, &addr, &size, VM_REGION_TOP_INFO, (vm_region_info_t)&info,
+      &info_count, &object_name);
+
+    if (kr == KERN_INVALID_ADDRESS) {
+      // Done iterating VM regions.
+      break;
+
+    } else if (kr != KERN_SUCCESS) {
+      ps__set_error(
+        "mach_vm_region(VM_REGION_TOP_INFO) syscall failed for %d",
+        (int) pid
+      );
+      ps__throw_error();
+    }
+
+    if (ps_in_shared_region(addr, cpu_type) &&
+        info.share_mode != SM_PRIVATE) {
+      continue;
+    }
+
+    switch (info.share_mode) {
+#ifdef SM_LARGE_PAGE
+    case SM_LARGE_PAGE:
+      // NB: Large pages are not shareable and always resident.
+#endif
+    case SM_PRIVATE:
+      private_pages += info.private_pages_resident;
+      private_pages += info.shared_pages_resident;
+      break;
+    case SM_COW:
+      private_pages += info.private_pages_resident;
+      if (info.ref_count == 1) {
+        // Treat copy-on-write pages as private if they only
+        // have one reference.
+        private_pages += info.shared_pages_resident;
+      }
+      break;
+    case SM_SHARED:
+    default:
+      break;
+    }
+  }
+
+  mach_port_deallocate(mach_task_self(), task);
+  return Rf_ScalarInteger(private_pages * pagesize);
+}
+
 SEXP ps__users() {
   struct utmpx *utx;
   SEXP result;
@@ -835,3 +960,238 @@ SEXP ps__users() {
   UNPROTECT(1);
   return result;
 }
+
+SEXP ps__disk_partitions(SEXP all) {
+  int num;
+  int i;
+  int len;
+  uint64_t flags;
+  char opts[400];
+  struct statfs *fs = NULL;
+  SEXP result;
+
+  // get the number of mount points
+  num = getfsstat(NULL, 0, MNT_NOWAIT);
+  if (num == -1) {
+    ps__set_error_from_errno();
+    goto error;
+  }
+
+  len = sizeof(*fs) * num;
+  fs = malloc(len);
+  if (fs == NULL) {
+    ps__no_memory("");
+    goto error;
+  }
+
+  num = getfsstat(fs, len, MNT_NOWAIT);
+  if (num == -1) {
+    ps__set_error_from_errno();
+    goto error;
+  }
+
+  PROTECT(result = allocVector(VECSXP, num));
+
+  for (i = 0; i < num; i++) {
+    opts[0] = 0;
+    flags = fs[i].f_flags;
+
+    // see sys/mount.h
+    if (flags & MNT_RDONLY)
+      strlcat(opts, "ro", sizeof(opts));
+    else
+      strlcat(opts, "rw", sizeof(opts));
+    if (flags & MNT_SYNCHRONOUS)
+      strlcat(opts, ",sync", sizeof(opts));
+    if (flags & MNT_NOEXEC)
+      strlcat(opts, ",noexec", sizeof(opts));
+    if (flags & MNT_NOSUID)
+      strlcat(opts, ",nosuid", sizeof(opts));
+    if (flags & MNT_UNION)
+      strlcat(opts, ",union", sizeof(opts));
+    if (flags & MNT_ASYNC)
+      strlcat(opts, ",async", sizeof(opts));
+    if (flags & MNT_EXPORTED)
+      strlcat(opts, ",exported", sizeof(opts));
+    if (flags & MNT_QUARANTINE)
+      strlcat(opts, ",quarantine", sizeof(opts));
+    if (flags & MNT_LOCAL)
+      strlcat(opts, ",local", sizeof(opts));
+    if (flags & MNT_QUOTA)
+      strlcat(opts, ",quota", sizeof(opts));
+    if (flags & MNT_ROOTFS)
+      strlcat(opts, ",rootfs", sizeof(opts));
+    if (flags & MNT_DOVOLFS)
+      strlcat(opts, ",dovolfs", sizeof(opts));
+    if (flags & MNT_DONTBROWSE)
+      strlcat(opts, ",dontbrowse", sizeof(opts));
+    if (flags & MNT_IGNORE_OWNERSHIP)
+      strlcat(opts, ",ignore-ownership", sizeof(opts));
+    if (flags & MNT_AUTOMOUNTED)
+      strlcat(opts, ",automounted", sizeof(opts));
+    if (flags & MNT_JOURNALED)
+      strlcat(opts, ",journaled", sizeof(opts));
+    if (flags & MNT_NOUSERXATTR)
+      strlcat(opts, ",nouserxattr", sizeof(opts));
+    if (flags & MNT_DEFWRITE)
+      strlcat(opts, ",defwrite", sizeof(opts));
+    if (flags & MNT_MULTILABEL)
+      strlcat(opts, ",multilabel", sizeof(opts));
+    if (flags & MNT_NOATIME)
+      strlcat(opts, ",noatime", sizeof(opts));
+    if (flags & MNT_UPDATE)
+      strlcat(opts, ",update", sizeof(opts));
+    if (flags & MNT_RELOAD)
+      strlcat(opts, ",reload", sizeof(opts));
+    if (flags & MNT_FORCE)
+      strlcat(opts, ",force", sizeof(opts));
+    if (flags & MNT_CMDFLAGS)
+      strlcat(opts, ",cmdflags", sizeof(opts));
+
+    SET_VECTOR_ELT(
+      result, i,
+      ps__build_list("ssss", fs[i].f_mntfromname, fs[i].f_mntonname,
+                     fs[i].f_fstypename, opts));
+  }
+
+  free(fs);
+  UNPROTECT(1);
+  return result;
+
+error:
+  if (fs != NULL) free(fs);
+  ps__throw_error();
+  return R_NilValue;
+}
+
+int ps__sys_vminfo(vm_statistics_data_t *vmstat) {
+  kern_return_t ret;
+  mach_msg_type_number_t count = sizeof(*vmstat) / sizeof(integer_t);
+  mach_port_t mport = mach_host_self();
+
+  ret = host_statistics(mport, HOST_VM_INFO, (host_info_t)vmstat, &count);
+  if (ret != KERN_SUCCESS) {
+    ps__set_error(
+      "host_statistics(HOST_VM_INFO) syscall failed: %s",
+      mach_error_string(ret)
+    );
+    return 1;
+  }
+  mach_port_deallocate(mach_task_self(), mport);
+  return 0;
+}
+
+SEXP ps__system_memory() {
+  int mib[2];
+  uint64_t total;
+  size_t len = sizeof(total);
+  vm_statistics_data_t vm;
+  int pagesize = getpagesize();
+  // physical mem
+  mib[0] = CTL_HW;
+  mib[1] = HW_MEMSIZE;
+
+  // This is also available as sysctlbyname("hw.memsize").
+  if (sysctl(mib, 2, &total, &len, NULL, 0)) {
+    if (errno != 0) {
+      ps__set_error_from_errno();
+    } else {
+      ps__set_error("sysctl(HW_MEMSIZE) syscall failed");
+    }
+    ps__throw_error();
+  }
+
+  // vm
+  if (ps__sys_vminfo(&vm)) ps__throw_error();
+
+  return ps__build_named_list(
+    "dddddd",
+    "total",       (double) total,
+    "active",      (double) vm.active_count * pagesize,
+    "inactive",    (double) vm.inactive_count * pagesize,
+    "wired",       (double) vm.wire_count * pagesize,
+    "free",        (double) vm.free_count * pagesize,
+    "speculative", (double) vm.speculative_count * pagesize
+  );
+}
+
+SEXP ps__system_swap() {
+  int mib[2];
+  size_t size;
+  struct xsw_usage totals;
+  vm_statistics_data_t vm;
+  int pagesize = getpagesize();
+
+  mib[0] = CTL_VM;
+  mib[1] = VM_SWAPUSAGE;
+  size = sizeof(totals);
+  if (sysctl(mib, 2, &totals, &size, NULL, 0) == -1) {
+  if (errno != 0) {
+      ps__set_error_from_errno();
+    } else {
+      ps__set_error("sysctl(VM_SWAPUSAGE) syscall failed");
+    }
+    ps__throw_error();
+  }
+
+  // vm
+  if (ps__sys_vminfo(&vm)) ps__throw_error();
+
+  return ps__build_named_list(
+    "ddddd",
+    "total", (double) totals.xsu_total,
+    "used",  (double) totals.xsu_used,
+    "free",  (double) totals.xsu_avail,
+    "sin",   (double) vm.pageins * pagesize,
+    "sout",  (double) vm.pageouts * pagesize
+  );
+}
+
+#define ARRAY_SIZE(a) (sizeof(a) / sizeof((a)[0]))
+
+SEXP ps__loadavg(SEXP counter_name) {
+  struct loadavg info;
+  size_t size = sizeof(info);
+  int which[] = {CTL_VM, VM_LOADAVG};
+
+  if (sysctl(which, ARRAY_SIZE(which), &info, &size, NULL, 0) < 0) {
+    ps__set_error_from_errno();
+    ps__throw_error();
+  }
+
+  SEXP ret = PROTECT(allocVector(REALSXP, 3));
+  REAL(ret)[0] = (double) info.ldavg[0] / info.fscale;
+  REAL(ret)[1] = (double) info.ldavg[1] / info.fscale;
+  REAL(ret)[2] = (double) info.ldavg[2] / info.fscale;
+
+  UNPROTECT(1);
+  return ret;
+}
+
+SEXP ps__system_cpu_times() {
+  mach_msg_type_number_t count = HOST_CPU_LOAD_INFO_COUNT;
+  kern_return_t error;
+  host_cpu_load_info_data_t r_load;
+
+  mach_port_t host_port = mach_host_self();
+  error = host_statistics(host_port, HOST_CPU_LOAD_INFO,
+                          (host_info_t)&r_load, &count);
+
+  mach_port_deallocate(mach_task_self(), host_port);
+
+  if (error != KERN_SUCCESS) {
+    ps__set_error_from_errno();
+    ps__throw_error();
+  }
+
+  const char *nms[] = { "user", "nice", "system", "idle", "" };
+  SEXP ret = PROTECT(Rf_mkNamed(REALSXP, nms));
+
+  REAL(ret)[0] = (double) r_load.cpu_ticks[CPU_STATE_USER]   / CLK_TCK;
+  REAL(ret)[1] = (double) r_load.cpu_ticks[CPU_STATE_NICE]   / CLK_TCK;
+  REAL(ret)[2] = (double) r_load.cpu_ticks[CPU_STATE_SYSTEM] / CLK_TCK;
+  REAL(ret)[3] = (double) r_load.cpu_ticks[CPU_STATE_IDLE]   / CLK_TCK;
+
+  UNPROTECT(1);
+  return ret;
+}
